package controllers

import (
	"encoding/json"
	"eta/eta_forum_hub/models"
	"eta/eta_forum_hub/services/alarm_msg"
	"eta/eta_forum_hub/utils"
	"fmt"
	"net/http"
	"net/url"

	"github.com/beego/beego/v2/server/web"
)

func init() {
}

type BaseAuthController struct {
	web.Controller
}

func (this *BaseAuthController) Prepare() {
	fmt.Println("enter prepare")
	method := this.Ctx.Input.Method()
	uri := this.Ctx.Input.URI()
	fmt.Println("Url:", uri)
	if method != "HEAD" {
		if method == "POST" || method == "GET" {
			authorization := this.Ctx.Input.Header("authorization")
			if authorization == "" {
				this.JSON(models.BaseResponse{Ret: 408, Msg: "请重新授权！", ErrMsg: "请重新授权:authorization is empty "}, false, false)
				this.StopRun()
				return
			}
			checkAuthorization := utils.MD5(utils.APP_NAME_EN + utils.Md5Key)
			fmt.Println(checkAuthorization)
			if authorization != checkAuthorization {
				this.JSON(models.BaseResponse{Ret: 408, Msg: "签名错误！", ErrMsg: "签名错误:authorization is err "}, false, false)
				this.StopRun()
				return
			}
		} else {
			this.JSON(models.BaseResponse{Ret: 408, Msg: "请求异常，请联系客服!", ErrMsg: "POST之外的请求，暂不支持"}, false, false)
			this.StopRun()
			return
		}
	} else {
		this.JSON(models.BaseResponse{Ret: 408, Msg: "请求异常，请联系客服!", ErrMsg: "method:" + method}, false, false)
		this.StopRun()
		return
	}
}

func (c *BaseAuthController) ServeJSON(encoding ...bool) {
	// 方法处理完后，需要后置处理的业务逻辑
	//if handlerList, ok := AfterHandlerUrlMap[c.Ctx.Request.URL.Path]; ok {
	//	for _, handler := range handlerList {
	//		handler(c.Ctx.Input.RequestBody)
	//	}
	//}

	var (
		hasIndent   = false
		hasEncoding = false
	)
	if web.BConfig.RunMode == web.PROD {
		hasIndent = false
	}
	if len(encoding) > 0 && encoding[0] == true {
		hasEncoding = true
	}
	if c.Data["json"] == nil {
		//go utils.SendEmail("异常提醒:", "接口:"+"URI:"+c.Ctx.Input.URI()+";无返回值", utils.EmailSendToUsers)
		body := "接口:" + "URI:" + c.Ctx.Input.URI() + ";无返回值"
		go alarm_msg.SendAlarmMsg(body, 1)
		return
	}

	baseRes := c.Data["json"].(*models.BaseResponse)
	if baseRes != nil && baseRes.Ret != 408 {
		body, _ := json.Marshal(baseRes)
		var requestBody string
		method := c.Ctx.Input.Method()
		if method == "GET" {
			requestBody = c.Ctx.Request.RequestURI
		} else {
			requestBody, _ = url.QueryUnescape(string(c.Ctx.Input.RequestBody))
		}
		if baseRes.Ret != 200 && baseRes.IsSendEmail {
			//go utils.SendEmail(utils.APPNAME+"【"+utils.RunMode+"】"+"失败提醒", "URI:"+c.Ctx.Input.URI()+"<br/> "+"Params"+requestBody+" <br/>"+"ErrMsg:"+baseRes.ErrMsg+";<br/>Msg:"+baseRes.Msg+";<br/> Body:"+string(body)+"<br/>"+c.SysUser.RealName, utils.EmailSendToUsers)
			tip := "URI:" + c.Ctx.Input.URI() + "<br/> " + "Params" + requestBody + " <br/>" + "ErrMsg:" + baseRes.ErrMsg + ";<br/>Msg:" + baseRes.Msg + ";<br/> Body:" + string(body)
			go alarm_msg.SendAlarmMsg(tip, 1)
		}
	}
	c.JSON(c.Data["json"], hasIndent, hasEncoding)
}

func (c *BaseAuthController) JSON(data interface{}, hasIndent bool, coding bool) error {
	c.Ctx.Output.Header("Content-Type", "application/json; charset=utf-8")
	var content []byte
	var err error
	if hasIndent {
		content, err = json.MarshalIndent(data, "", "  ")
	} else {
		content, err = json.Marshal(data)
	}
	if err != nil {
		http.Error(c.Ctx.Output.Context.ResponseWriter, err.Error(), http.StatusInternalServerError)
		return err
	}
	ip := c.Ctx.Input.IP()
	requestBody, err := url.QueryUnescape(string(c.Ctx.Input.RequestBody))
	if err != nil {
		requestBody = string(c.Ctx.Input.RequestBody)
	}
	if requestBody == "" {
		requestBody = c.Ctx.Input.URI()
	}
	authorization := c.Ctx.Input.Header("authorization")
	if authorization == "" {
		authorization = c.Ctx.Input.Header("Authorization")
	}
	uri := c.Ctx.Input.URI()
	if uri == `/v1/chart/update` || uri == `/v1/chart/save` {
		utils.ApiLog.Info("uri:%s, authorization:%s, requestBody:%s, responseBody:%s, ip:%s", uri, authorization, "", content, ip)
	} else {
		utils.ApiLog.Info("uri:%s, authorization:%s, requestBody:%s, responseBody:%s, ip:%s", uri, authorization, requestBody, content, ip)
	}
	if coding {
		content = []byte(utils.StringsToJSON(string(content)))
	}
	return c.Ctx.Output.Body(content)
}
