package controllers

import (
	"encoding/json"
	"eta/eta_forum_hub/models"
	"eta/eta_forum_hub/services"
)

type ChartClassifyController struct {
	BaseAuthController
}

// Save 新增或者更新分类
// @Title 新增或者更新分类
// @Description 新增或者更新分类
// @Param   request	body models.ChartClassifySyncReq true "type json string"
// @Success 200 {object} models.BaseResponse
// @router /save [post]
func (this *ChartClassifyController) Save() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	
	var req models.ChartClassify
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.ChartClassifyId == 0 {
		br.Msg = "请输入分类id"
		br.ErrMsg = "请输入分类id"
		return
	}
	if req.IsSelected != 1 {
		br.Msg = "只有精选资源才可以更新"
		return
	}
	err = services.AddOrUpdateChartClassify(&req)
	if err != nil {
		br.Msg = "新增或者更新分类失败"
		br.ErrMsg = "新增或者更新分类失败, Err:" + err.Error()
		return
	}
	
	br.Ret = 200
	br.Success = true
	br.Msg = "新增成功"
}

// BatchSave 批量同步分类
// @Title 批量同步分类
// @Description 批量同步分类
// @Param   request	body models.ChartClassifySyncReq true "type json string"
// @Success 200 {object} models.BaseResponse
// @router /batch_save [post]
func (this *ChartClassifyController) BatchSave() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	
	var req models.ChartClassifySyncReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if len(req.List) == 0 {
		br.Msg = "请输入列表"
		br.ErrMsg = "请输入列表"
		return
	}
	err, isSendEmail := services.BatchAddOrUpdateChartClassify(&req)
	if err != nil {
		br.Msg = "新增或者更新用户失败"
		br.ErrMsg = "新增或者更新用户失败, Err:" + err.Error()
		br.IsSendEmail = isSendEmail
		return
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "新增成功"
	
}