package system

import (
	"encoding/json"
	"eta/eta_forum_hub/controllers"
	"eta/eta_forum_hub/models"
	"eta/eta_forum_hub/models/system"
	"eta/eta_forum_hub/services"
)

type SysRoleController struct {
	controllers.BaseAuthController
}

// Save
// @Title 新增或者编辑系统用户
// @Description 新增系统用户接口
// @Param	request	body system.SysuserAddReq true "type json string"
// @Success 200 新增成功
// @router /role/save [post]
func (this *SysRoleController) Save() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req system.AddRoleReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}

	if req.RoleId == 0 {
		br.Msg = "请选择角色"
		return
	}

	err, isSendEmail := services.AddOrUpdateRole(&req)
	if err != nil {
		br.Msg = "新增或者更新用户失败"
		br.ErrMsg = "新增或者更新用户失败, Err:" + err.Error()
		br.IsSendEmail = isSendEmail
		return
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "新增成功"
}

// Delete
// @Title 删除系统用户
// @Description 删除系统用户接口
// @Param	request	body system.SysRoleDeleteReq true "type json string"
// @Success 200 删除成功
// @router /role/delete [post]
func (this *SysRoleController) Delete() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req system.RoleDeleteReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.RoleId == 0 {
		br.Msg = "请选择角色"
		return
	}

	err = system.DeleteSysRole(req.RoleId)
	if err != nil {
		br.Msg = "删除失败"
		br.ErrMsg = "删除失败，Err:" + err.Error()
		return
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "删除成功"
}

// Sync
// @Title 新增或者编辑系统用户
// @Description 新增系统用户接口
// @Param	request	body system.SysuserAddReq true "type json string"
// @Success 200 新增成功
// @router /role/sync [post]
func (this *SysRoleController) Sync() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req system.SysRoleSyncReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if len(req.List) == 0 {
		br.Msg = "请输入组合"
		br.IsSendEmail = false
		return
	}

	err, isSendEmail := services.SyncSysRole(&req)
	if err != nil {
		br.Msg = "新增或者更新角色失败"
		br.ErrMsg = "新增或者更新角色失败, Err:" + err.Error()
		br.IsSendEmail = isSendEmail
		return
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "新增成功"
}
