package system

import (
	"encoding/json"
	"errors"
	"eta/eta_forum_hub/utils"
	"github.com/beego/beego/v2/client/orm"
)

const (
	ConfAreaCodeListKey         = "area_code_list" // 手机号区号列表
	CrmConfigLoginSmsTpId       = "LoginSmsTpId"
	CrmConfigLoginSmsGjTpId     = "LoginSmsGjTpId"
	CrmConfigSmsJhgnAppKey      = "SmsJhgnAppKey"
	CrmConfigSmsJhgjAppKey      = "SmsJhgjAppKey"
	CrmConfigLoginSmsTplContent = "LoginSmsTplContent"
	CrmConfigSmsJhgjVariable    = "SmsJhgjVariable" // 聚合国际短信变量
	CrmConfigICPLicense         = "ICPLicense"
	CrmConfigLogoCN             = "LogoCN"
	CrmConfigLogoCNMini         = "LogoCNMini"
	CrmConfigCompanyWatermark   = "CompanyWatermark"
)

type CrmConfig struct {
	ConfigCode  string `description:"详情Code"`
	ConfigValue string `description:"详情"`
}

func GetConfigValueByCode(configCode string) (total int, err error) {
	o := orm.NewOrm()
	sql := ` SELECT config_value FROM crm_config WHERE config_code=? `
	err = o.Raw(sql, configCode).QueryRow(&total)
	return
}

// 修改
func CrmConfigUpdate(newValue, configCode string) (err error) {
	o := orm.NewOrm()
	sql := `UPDATE crm_config SET  config_value=?   WHERE config_code=  ?`
	_, err = o.Raw(sql, newValue, configCode).Exec()
	return
}

// ConfigClassifyId
// @Description: 后台配置的报告id
type ConfigClassifyId struct {
	Debug   int `json:"debug"`
	Release int `json:"release"`
}

func GetCrmConfigDetailByCode(configCode string) (item *CrmConfig, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM crm_config WHERE config_code=? `
	err = o.Raw(sql, configCode).QueryRow(&item)
	return
}

// GetCrmConfig 获取基础配置
func GetCrmConfig() (list map[string]string, err error) {
	list = make(map[string]string)

	var items []*CrmConfig
	o := orm.NewOrm()
	sql := `SELECT * FROM crm_config`
	_, err = o.Raw(sql).QueryRows(&items)
	if err != nil {
		return
	}

	for _, v := range items {
		list[v.ConfigCode] = v.ConfigValue
	}
	return
}

// GetReportClassifyIdByConfigKey
// @Description: 获取关联的报告id
// @author: Roc
// @datetime 2024-06-18 14:10:27
// @param configKey string
// @return classifyId int
// @return err error
func GetReportClassifyIdByConfigKey(configKey string) (classifyId int, err error) {
	// 别问为啥要从配置里拿=_=!
	conf, e := GetCrmConfigDetailByCode(configKey)
	if e != nil {
		err = errors.New("获取配置的id失败, Err: " + e.Error())
		return
	}
	if conf.ConfigValue == "" {
		err = errors.New("ID配置有误")
		return
	}
	type TwoWeekIdConf struct {
		Debug   []int
		Release []int
	}
	classifyIdConf := new(ConfigClassifyId)
	if e = json.Unmarshal([]byte(conf.ConfigValue), &classifyIdConf); e != nil {
		err = errors.New("解析ID配置失败, Err: " + e.Error())
		return
	}
	if utils.RunMode == "debug" {
		classifyId = classifyIdConf.Debug
	} else {
		classifyId = classifyIdConf.Release
	}

	return
}

type BaseInfoResp struct {
	Icp string `description:"Icp信息"`
	//ETATitle   *CrmConfig `description:"eta系统名称"`
	//	TabName *CrmConfig `description:"tab页名称"`
	LogoCn string `description:"中文logo"`
	//LogoEn     *CrmConfig `description:"英文logo"`
	LogoCnMini string `description:"中文logoMini"`
	//	LogoEnMini *CrmConfig `description:"英文logoMini"`
}
