package system

import (
	"github.com/beego/beego/v2/client/orm"
)

type SysRole struct {
	RoleId       int    `orm:"column(role_id);pk" description:"角色ID"`
	RoleName     string `description:"角色名称"`
	RoleType     string `description:"角色类型"`
	RoleTypeCode string `description:"角色类型编码"`
	RoleLevel    int    `description:"角色等级：0-表示一级角色，每个账号只能绑定一个一级角色 ，1-表示二级角色，每个账号可以绑定多个二级 角色"`
	CreateTime   string `description:"创建时间"`
	ModifyTime   string
}

type AddRoleReq struct {
	RoleId       int    `description:"角色ID"`
	RoleName     string `description:"角色名称"`
	RoleType     string `description:"角色类型"`
	RoleTypeCode string `description:"角色类型编码"`
	RoleLevel    int    `description:"角色等级：0-表示一级角色，每个账号只能绑定一个一级角色 ，1-表示二级角色，每个账号可以绑定多个二级 角色"`
	CreateTime   string `description:"创建时间"`
	ModifyTime   string
	BusinessCode string `description:"商家编码"`
}

func GetSysRoleByRoleId(roleId int) (item *SysRole, err error) {
	sql := `SELECT * FROM sys_role WHERE role_id=? `
	o := orm.NewOrm()
	err = o.Raw(sql, roleId).QueryRow(&item)
	return
}

func AddSysRole(item *SysRole) (lastId int64, err error) {
	o := orm.NewOrm()
	lastId, err = o.Insert(item)
	return
}

// Update 更新用户基础信息
func (item *SysRole) Update(cols []string) (err error) {
	o := orm.NewOrm()
	_, err = o.Update(item, cols...)
	return
}

type RoleDeleteReq struct {
	RoleId       int
	BusinessCode string `description:"商家编码"`
}

func DeleteSysRole(roleId int) (err error) {
	sql := `DELETE FROM sys_role WHERE role_id=? `
	o := orm.NewOrm()
	_, err = o.Raw(sql, roleId).Exec()
	return
}

type SysRoleSyncReq struct {
	List []*SysRole
}

// 获取所有角色信息
func GetSysRoleList() (list []*SysRole, err error) {
	sql := `SELECT * FROM sys_role `
	o := orm.NewOrm()
	_, err = o.Raw(sql).QueryRows(&list)
	return
}

func (item *SysRole) AddOrUpdate() (err error) {
	o := orm.NewOrm()
	_, err = o.InsertOrUpdate(item)
	return
}
