package eta_trial

import (
	"eta/eta_forum_hub/models/eta_trial"
	"eta/eta_forum_hub/services/alarm_msg"
	"eta/eta_forum_hub/utils"
	"fmt"
	"time"
)

func EtaTrialManualUserAddAuth(adminId int, adminName string) (err error) {
	if adminId <= 0 {
		err = fmt.Errorf("CreateManualAuth参数有误, AdminId: %d", adminId)
		return
	}

	// 从ETA试用平台的edb库中读取手工数据分类
	classifyIds := make([]int, 0)
	{
		classifyOb := new(eta_trial.EdbdataClassify)
		cond := `is_show = 1`
		pars := make([]interface{}, 0)
		list, e := classifyOb.GetItemsByCondition(cond, pars, []string{}, "")
		if e != nil {
			err = fmt.Errorf("操作失败", "获取手工数据分类失败, Err: "+e.Error())
			return
		}
		for _, v := range list {
			classifyIds = append(classifyIds, v.ClassifyId)
		}
	}

	// 新增用户及权限
	manualUser := new(eta_trial.ManualUser)
	manualUser.AdminId = adminId
	manualUser.AdminRealName = adminName
	manualUser.CreateTime = time.Now().Local()
	if e := manualUser.Create(); e != nil {
		err = fmt.Errorf("操作失败", "新增权限用户失败, Err: "+e.Error())
		return
	}
	userClassifies := make([]*eta_trial.ManualUserClassify, 0)
	for _, v := range classifyIds {
		userClassifies = append(userClassifies, &eta_trial.ManualUserClassify{
			AdminId:    adminId,
			ClassifyId: v,
			CreateTime: time.Now().Local(),
		})
	}
	userClassifyOb := new(eta_trial.ManualUserClassify)
	if e := userClassifyOb.BatchCreate(userClassifies); e != nil {
		err = fmt.Errorf("操作失败", "批量新增权限用户分类失败, Err: "+e.Error())
		return
	}
	return
}

// ETA试用客户 到期自动禁用
func ETATrialAutoDisabled() (err error) {
	defer func() {
		if err != nil {
			go alarm_msg.SendAlarmMsg("ETA试用客户 启用->禁用 ErrMsg:"+err.Error(), 3)
		}
	}()

	expiredTime := time.Now().AddDate(0, 0, -14).Format(utils.FormatDate)

	items, err := eta_trial.GetETATrialDisableList(expiredTime)
	if err != nil && err.Error() != utils.ErrNoRow() {
		fmt.Println("GetETATrialDisableList Err:" + err.Error())
		utils.FileLog.Info("GetETATrialDisableList Err:%s" + err.Error())
		return
	}
	disableIds := make([]int, 0)
	disableMobiles := make([]string, 0)
	for _, v := range items {
		disableIds = append(disableIds, v.EtaTrialId)
		disableMobiles = append(disableMobiles, v.Mobile)
	}

	if len(disableIds) > 0 {
		err = eta_trial.UpdateETATrialDisable(disableIds)
		if err != nil {
			return err
		}
	}
	if len(disableMobiles) > 0 {
		err = eta_trial.UpdateETATrialAdminDisable(disableMobiles)
		if err != nil {
			return err
		}
	}

	return
}
